const pause = async (pool, readModelName) => {
  const {
    PassthroughError,
    inlineLedgerRunQuery,
    inlineLedgerForceStop,
    tablePrefix,
    escapeId,
    escapeStr
  } = pool;
  const ledgerTableNameAsId = escapeId(`${tablePrefix}__LEDGER__`);

  try {
    pool.activePassthrough = true;

    while (true) {
      try {
        await inlineLedgerForceStop(pool, readModelName);
        await inlineLedgerRunQuery(`BEGIN TRANSACTION;
        
         SELECT * FROM ${ledgerTableNameAsId}
         WHERE \`EventSubscriber\` = ${escapeStr(readModelName)}
         FOR UPDATE NOWAIT;

         UPDATE ${ledgerTableNameAsId}
         SET \`IsPaused\` = TRUE
         WHERE \`EventSubscriber\` = ${escapeStr(readModelName)};

         COMMIT;
      `);
        break;
      } catch (err) {
        if (!(err instanceof PassthroughError)) {
          throw err;
        }
      }
    }
  } finally {
    pool.activePassthrough = false;
  }
};

export default pause;
//# sourceMappingURL=pause.js.map