import type { CommonAdapterPool, CommonAdapterOptions, AdapterOperations, AdapterConnection, AdapterImplementation, StoreApi, PerformanceTracerLike, SplitNestedPathMethod, JsonMap, SearchCondition, UpdateCondition, OmitObject } from '@resolve-js/readmodel-base';
import type MySQLPromiseLib from 'mysql2/promise';
import type { escape as _EscapeableMethod } from 'mysql2';
export * from '@resolve-js/readmodel-base';
export declare type EscapeableMethod = typeof _EscapeableMethod;
export declare type LibDependencies = {
    MySQL: typeof MySQLPromiseLib;
};
export declare type InlineLedgerRunQueryMethod = (querySQL: string, passthroughRuntimeErrors?: boolean) => Promise<Array<object>>;
export declare type FullJitterMethod = (retries: number) => Promise<void>;
export declare type MakeNestedPathMethod = (nestedPath: Array<string>) => string;
export declare type InlineLedgerForceStopMethod = (pool: AdapterPool, readModelName: string) => Promise<void>;
export declare type BuildUpsertDocumentMethod = (searchExpression: Parameters<StoreApi<CommonAdapterPool>['update']>[3], updateExpression: Parameters<StoreApi<CommonAdapterPool>['update']>[4], splitNestedPath: SplitNestedPathMethod) => JsonMap;
export declare type RowLike = JsonMap;
export declare type MarshalledRowLike = RowLike & {
    marshalled: 'marshalled';
};
export declare type ConvertBinaryRowMethod = (inputRow: MarshalledRowLike, fieldList: Parameters<StoreApi<CommonAdapterPool>['find']>[4] | null) => RowLike;
export declare type SearchToWhereExpressionMethod = (expression: SearchCondition, escapeId: EscapeableMethod, escapeStr: EscapeableMethod, makeNestedPath: MakeNestedPathMethod, splitNestedPath: SplitNestedPathMethod) => string;
export declare type UpdateToSetExpressionMethod = (expression: UpdateCondition, escapeId: EscapeableMethod, escapeStr: EscapeableMethod, makeNestedPath: MakeNestedPathMethod, splitNestedPath: SplitNestedPathMethod) => string;
export interface PassthroughErrorInstance extends Error {
    name: string;
}
export declare type PassthroughErrorFactory = {
    new (): PassthroughErrorInstance;
} & {
    isPassthroughError: (error: Error & {
        errno: string | number;
    }, includeRuntimeErrors: boolean) => boolean;
};
export declare type GenerateGuidMethod = (...args: any) => string;
export declare type DropReadModelMethod = (pool: AdapterPool, readModelName: string) => Promise<void>;
export declare type AdapterOptions = CommonAdapterOptions & {
    tablePrefix?: string;
} & MySQLPromiseLib.ConnectionOptions;
export declare type MaybeInitMethod = (pool: AdapterPool) => Promise<void>;
export declare type InternalMethods = {
    inlineLedgerForceStop: InlineLedgerForceStopMethod;
    buildUpsertDocument: BuildUpsertDocumentMethod;
    convertBinaryRow: ConvertBinaryRowMethod;
    searchToWhereExpression: SearchToWhereExpressionMethod;
    updateToSetExpression: UpdateToSetExpressionMethod;
    PassthroughError: PassthroughErrorFactory;
    generateGuid: GenerateGuidMethod;
    dropReadModel: DropReadModelMethod;
    escapeId: EscapeableMethod;
    escapeStr: EscapeableMethod;
    maybeInit: MaybeInitMethod;
};
export declare type AdapterPool = CommonAdapterPool & {
    inlineLedgerRunQuery: InlineLedgerRunQueryMethod;
    fullJitter: FullJitterMethod;
    connectionOptions: Omit<Omit<AdapterOptions, 'tablePrefix'>, 'databaseFile'>;
    performanceTracer: PerformanceTracerLike;
    tablePrefix: string;
    databaseFile: string;
    makeNestedPath: MakeNestedPathMethod;
    getConnection: () => Promise<MySQLPromiseLib.Connection>;
    connection: MySQLPromiseLib.Connection;
    activePassthrough: boolean;
    distinctMode: boolean;
} & {
    [K in keyof AdapterOperations<CommonAdapterPool>]: AdapterOperations<AdapterPool>[K];
} & {
    [K in keyof StoreApi<CommonAdapterPool>]: StoreApi<AdapterPool>[K];
} & InternalMethods;
export declare type CurrentAdapterConnection = AdapterConnection<AdapterPool, OmitObject<AdapterOptions, CommonAdapterPool>>;
export declare type ExternalMethods = {
    [K in keyof AdapterOperations<CommonAdapterPool>]: AdapterPool[K];
};
export declare type CurrentStoreApi = {
    [K in keyof StoreApi<CommonAdapterPool>]: AdapterPool[K];
};
export declare type ConnectionDependencies = LibDependencies & InternalMethods & ExternalMethods & CurrentStoreApi;
export declare type CurrentConnectMethod = (imports: ConnectionDependencies, ...args: Parameters<CurrentAdapterConnection['connect']>) => ReturnType<CurrentAdapterConnection['connect']>;
export declare type CurrentDisconnectMethod = CurrentAdapterConnection['disconnect'];
export declare type CurrentAdapterImplementation = AdapterImplementation<AdapterPool, AdapterOptions>;
export declare type AdminOptions = MySQLPromiseLib.ConnectionOptions & {
    database: string;
};
export declare type BoundResourceMethod = (options: AdminOptions) => Promise<void>;
export declare type UnboundResourceMethod = (pool: AdminPool, options: AdminOptions) => Promise<void>;
export declare type AdminPool = {
    connect: CurrentAdapterImplementation['connect'];
    disconnect: CurrentAdapterImplementation['disconnect'];
    escapeStr: EscapeableMethod;
    escapeId: EscapeableMethod;
    createResource: BoundResourceMethod;
    destroyResource: BoundResourceMethod;
};
